<?php

namespace Drupal\Tests\webform_remote_fields\Functional;

/**
 * Test access on webform_remote_field plugins operations.
 *
 * @group webform_remote_fields
 */
class WebformRemoteFieldConfigurationUiTest extends WebformRemoteFieldsTestBase {

  /**
   * The name of the Webform to use on the tests.
   *
   * @var string
   */
  protected $webformName = 'test_webform';

  /**
   * The name of the field to test.
   *
   * @var string
   */
  protected $fieldName = 'number_remote_field_test';

  /**
   * Test if user with properly permissions are able to do all operations.
   */
  public function testUserWithPermissions() {
    $webform = $this->loadWebform('select');

    // Testing creating element.
    $this->drupalGet("/admin/structure/webform/manage/{$webform}/element/add/remote_field_number");

    $this->assertSession()
      ->elementExists('css', 'input[name="properties[api_endpoint]"]');
    $this->assertSession()
      ->elementExists('css', 'input[name="properties[api_result_select_value]"]');

    $this->submitForm(
      [
        'properties[title]' => 'number_remote_field_test',
        'properties[api_endpoint]' => 'http://example.com',
        'properties[api_result_select_value]' => 'value',
      ],
      'Save'
    );
    $this->assertTrue($this->assertSession()->waitForText("{$this->fieldName} has been created.", 2000));
  }

  /**
   * Test if user without the properly permissions receive access denied.
   */
  public function testUserWithoutPermission() {
    $admin_user = $this->drupalCreateUser([
      'create webform',
      'edit any webform',
    ]);
    $this->drupalLogin($admin_user);
    $webform = $this->loadWebform('select');

    // Testing creating an element.
    $this->drupalGet("/admin/structure/webform/manage/{$webform}/element/add/remote_field_number");
    $this->assertSession()->pageTextContains('You do not have access to edit this field.');
    $this->assertSession()
      ->elementNotExists('css', 'input[name="properties[api_endpoint]"]');
    $this->assertSession()
      ->elementNotExists('css', 'input[name="properties[api_result_select_value]"]');

    // Testing editing element.
    $this->drupalGet("/admin/structure/webform/manage/{$webform}/element/remote_select_1/edit");
    $this->assertTrue($this->assertSession()->waitForText("You do not have access to edit this field."));

    $this->drupalGet("/admin/structure/webform/manage/{$webform}/element/remote_select_1/delete");
    $this->assertTrue($this->assertSession()->waitForText("You do not have access to delete this field."));

    // Testing delete handler.
    $this->drupalGet("/admin/structure/webform/manage/{$webform}/handlers/webform_remote_fields_handler/delete");
    $this->assertTrue(
      $this->assertSession()->waitForText("You do not have access to delete this handler."),
      'The message about not being able to delete the handler is displayed.'
    );

    // Testing disable handler.
    $this->drupalGet("/admin/structure/webform/manage/{$webform}/handlers");
    $page = $this->getSession()->getPage();
    $ul = $page->find('css', "ul[data-drupal-selector='edit-handlers-webform-remote-fields-handler-operations']");
    $ul->find('css', "li.dropbutton-toggle button")->click();
    $ul->clickLink('Disable');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertTrue(
      $this->assertSession()->waitForText("You don't have access to disable this handler!"),
      'The message about not being able to disable the handler is displayed.'
    );
  }

}
