<?php

namespace Drupal\Tests\webform_remote_fields\FunctionalJavascript;

use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the remote fields' functionality.
 *
 * @group webform
 * @group webform_remote_fields
 */
class WebformRemoteFieldTest extends WebDriverTestBase {

  /**
   * {@inheritDoc}
   */
  protected static $modules = [
    'webform',
    'webform_ui',
    'webform_test_element',
    'webform_remote_fields',
    'webform_remote_fields_api_tests_helper',
  ];

  /**
   * {@inheritDoc}
   */
  protected $defaultTheme = 'stable9';

  /**
   * Test basic features of remote fields.
   */
  public function testAllRemoteFieldsElements() {
    $this->drupalGet('/webform/remote_field_example_basic');
    $this->assertSession()->pageTextContains('Remote field example basic');

    $page = $this->getSession()->getPage();

    $productElement = $page->findField('list_fake_products');
    $priceElement = $page->findField('price');
    $quantityElement = $page->findField('quantity');
    $discountElement = $page->findField('discount');

    // Select a product and only price should be updated.
    $productElement->selectOption('2');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $firstPrice = $priceElement->getValue();
    $this->assertNotEmpty($firstPrice);
    $this->assertEmpty($discountElement->getValue());

    // Change quantity.
    $quantityElement->setValue(2);
    $this->assertSession()->assertWaitOnAjaxRequest();
    $firstDiscount = $discountElement->getValue();
    $this->assertIsNumeric($firstDiscount);

    // Change quantity second time.
    $quantityElement->setValue(5);
    $this->assertSession()->assertWaitOnAjaxRequest();
    $secondDiscount = $discountElement->getValue();
    $this->assertIsNumeric($secondDiscount);
    $this->assertNotEquals($firstDiscount, $secondDiscount);

    // Change product price.
    $productElement->selectOption('1');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $secondPrice = $priceElement->getValue();
    $this->assertNotEmpty($secondPrice);
    $this->assertNotEquals($firstPrice, $secondPrice);
  }

  /**
   * Test remote fields with multiple steps.
   */
  public function testMultipleSteps() {
    $this->drupalGet('/webform/remote_field_multiple_step');
    $this->assertSession()->pageTextContains('Remote Field Multiple Step Example');
    $page = $this->getSession()->getPage();

    $page->findField('list_fake_products')->selectOption(2);
    $this->assertSession()->assertWaitOnAjaxRequest();
    $firstPrice = $page->findField('price')->getValue();
    $this->assertNotEmpty($firstPrice);

    $page->findButton('Next')->click();
    $quantity = $this->assertSession()->waitForElement('css', 'input[name="quantity"]');
    // At this point, the only field that depends on the quantity is not
    // present on the page, so any ajax request should not be triggered.
    // So, we need to check if the form_build_id still the same after
    // changing the quantity.
    $formBuild = $page->find('css', 'input[name="form_build_id"]')->getValue();
    $quantity->setValue(1);
    $quantity->setValue(5);
    $this->assertEquals($formBuild, $page->find('css', 'input[name="form_build_id"]')->getValue());
    // Now, we have a size field that depends on the color fields,
    // since they change the color field, the size field should be updated,
    // and the form_build_id should be different.
    $page->find('css', '#edit-colors-and-size')->click();
    $page->findField('edit-colors-available-fff')->selectOption('FFF');
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertNotEquals($formBuild, $page->find('css', 'input[name="form_build_id"]')->getValue());
    $page->findField('size')->selectOption('g');

    $page->findButton('Previous')->click();
    $page->findField('list_fake_products')->selectOption(1);
    $this->assertSession()->assertWaitOnAjaxRequest();
    $this->assertNotEmpty($page->findField('price')->getValue(), $firstPrice);

    $page->findButton('Next')->click();
    $quantity = $this->assertSession()->waitForElement('css', 'input[name="quantity"]');
    $this->assertEquals("5", $quantity->getValue());

    $page->findButton('Next')->click();
    $discount = $this->assertSession()->waitForElement('css', 'input[name="discount"]');
    $this->assertNotEmpty($discount->getValue());
  }

}
