<?php

namespace Drupal\Tests\webform_remote_fields\UnitTest;

use PHPUnit\Framework\TestCase;
use Drupal\webform_remote_fields\Traits\RemoteFieldBaseTrait;

/**
 * Test trait RemoteFieldBaseTrait.
 *
 * @covers \Drupal\webform_remote_fields\Traits\RemoteFieldBaseTrait
 * @group webform_remote_fields
 */
class RemoteFieldBaseTraitTest extends TestCase {

  use RemoteFieldBaseTrait;

  /**
   * Data provider for testGetDataFromResponse.
   *
   * @return \Generator
   *   Test cases with response, root_property_path, and expected value.
   */
  public static function dataProviderForGetDataFromResponse(): \Generator {
    // Valid path with single depth.
    yield 'Valid path with single depth.' => [
      ['results' => ['approved' => 'success']],
      'results.approved',
      'success',
    ];

    // Valid path with multiple depths.
    yield 'Valid path with multiple depths.' => [
      ['results' => ['approved' => ['value' => 'success']]],
      'results.approved.value',
      'success',
    ];

    // Root property path.
    yield 'Root property path.' => [
      ['results' => ['approved' => ['value' => 'success']]],
      'results',
      ['approved' => ['value' => 'success']],
    ];

    // Non-existent path.
    yield 'Non-existent path.' => [
      ['results' => ['approved' => ['value' => 'success']]],
      'results.rejected.value',
      [],
    ];

    // Empty response.
    yield 'Empty response.' => [
      [],
      'results.approved.value',
      [],
    ];

    // Empty root_property_path.
    yield 'Empty root_property_path.' => [
      ['results' => ['approved' => ['value' => 'success']]],
      '',
      ['results' => ['approved' => ['value' => 'success']]],
    ];
  }

  /**
   * Test the getDataFromResponse method with multiple cases.
   *
   * @dataProvider dataProviderForGetDataFromResponse
   */
  public function testGetDataFromResponse(array $response, string $root_property_path, $expected) {
    $result = self::getDataFromResponse($root_property_path, $response);
    $this->assertEquals($expected, $result);
  }

}
